unit TransparentGroupBox;

{
TTransparentGroupBox

TGroupBox transparent, on peut voir les TImage et le canvas
de la fiche  travers le GroupBox

Adrien Montfaucon - Vector

Mises  jour : http://www.phidels.com ou http://admon.free.fr
Suggestions : am-vector@laposte.net
}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls, Forms,
  Graphics, ExtCtrls;

type
  TTransparentGroupBox = class(TGroupBox)
  private
    FParentForm:TForm;
    FTransparent: Boolean;
    FDoubleBuffer:TBitmap;
    FOptimize: Boolean;
    function GetParentForm(Child:TComponent):TForm;
    procedure SetTransparent(IsTransparent:Boolean);
    function CopyControlsImage(Parent:TForm):TBitmap;
    procedure CreateDoubleBuffer;
    procedure CustomPaint;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    procedure RefreshData;
  published
    property Transparent:Boolean read FTransparent write SetTransparent;
    property Optimize:Boolean read FOptimize write FOptimize;
  end;

procedure Register;

implementation

{Enregistrement du composants}

procedure Register;
begin
RegisterComponents('Admon', [TTransparentGroupBox]);
end;

{ TTransparentGroupBox }

function TTransparentGroupBox.CopyControlsImage(
  Parent: TForm): TBitmap;
var I:Word;
    ChildImage:TImage;
begin
Result:=TBitmap.Create;
Result.Height:=FParentForm.ClientHeight;
Result.Width:=FParentForm.ClientWidth;
Result.Canvas.Brush.Color:=Parent.Color;
Result.Canvas.FillRect(Parent.ClientRect);
for I:=0 to Parent.ComponentCount-1 do
  begin
  if (Parent.Components[I] is TImage) then
    begin
    ChildImage:=(Parent.Components[I] as TImage);
    if ChildImage.Stretch then
      Result.Canvas.StretchDraw(ChildImage.BoundsRect,ChildImage.Picture.Graphic)
    else
      Result.Canvas.Draw(ChildImage.Left,ChildImage.Top,ChildImage.Picture.Graphic);
    end;
  end;
end;

constructor TTransparentGroupBox.Create(AOwner: TComponent);
begin
inherited;
FTransparent:=True;
FDoubleBuffer:=TBitmap.Create;
FOptimize:=True;
RefreshData;
end;

procedure TTransparentGroupBox.CreateDoubleBuffer;
begin
FDoubleBuffer:=CopyControlsImage(FParentForm);
end;

procedure TTransparentGroupBox.CustomPaint;
var H,XSource,YSource:Integer;
    R:TRect;
    Flags:Longint;
begin
with Canvas do
  begin
  BitBlt(Handle,0,0,ClipRect.Right,ClipRect.Bottom,FDoubleBuffer.Canvas.Handle,Left,Top,SRCCOPY);
  Font:=Self.Font;
  H:=TextHeight('0');
  R:=Rect(0,H div 2-1, Width,Height);
  if Ctl3D then
    begin
    Inc(R.Left);
    Inc(R.Top);
    Brush.Color:=clBtnHighlight;
    FrameRect(R);
    OffsetRect(R,-1,-1);
    Brush.Color:=clBtnShadow;
    end
  else
    Brush.Color:=clWindowFrame;
  FrameRect(R);
  if Text<>'' then
    begin
    if not UseRightToLeftAlignment then
      R:=Rect(8,0,0,H)
    else
      R:=Rect(R.Right-Canvas.TextWidth(Text)-8,0,0,H);
    XSource:=ClientToParent(Point(8,0),FParentForm).X;
    YSource:=ClientToParent(Point(8,0),FParentForm).Y;
    BitBlt(Handle,8,0,TextWidth(Text),H,FDoubleBuffer.Canvas.Handle,XSource,YSource,SRCCOPY);
    Flags:=DrawTextBiDiModeFlags(DT_SINGLELINE);
    DrawText(Handle,PChar(Text),Length(Text),R,Flags or DT_CALCRECT);
    Brush.Style:=bsClear;
    DrawText(Handle,PChar(Text),Length(Text),R,Flags);
    end;
  end;
end;

destructor TTransparentGroupBox.Destroy;
begin
FreeAndNil(FDoubleBuffer);
inherited;
end;

function TTransparentGroupBox.GetParentForm(Child: TComponent): TForm;
begin
if Child.Owner is TForm then Result:=Child.Owner as TForm
else Result:=GetParentForm(Child.Owner) as TForm;
end;

procedure TTransparentGroupBox.Paint;
begin
if (not Transparent) or (csDesigning in ComponentState) then
  inherited
else
  begin
  CustomPaint;
  if not FOptimize then
    begin
    CreateDoubleBuffer;
    CustomPaint;
    end;
  end;
end;

procedure TTransparentGroupBox.RefreshData;
begin
FParentForm:=GetParentForm(Self);
CreateDoubleBuffer;
end;

procedure TTransparentGroupBox.SetTransparent(IsTransparent: Boolean);
begin
if FTransparent<>IsTransparent then
  begin
  FTransparent:=IsTransparent;
  Invalidate;
  end;
end;

end.
